/*
 * filename: src/gutils.c
 * author: Pablo Yanez Trujillo <yanezp@imtek.de>
 */

/* This file is an extension of the GTK+ GLib.
 * This file contains functions that the GTK+ GLib does not
 * contain on their official releases (but should have)
 *
 * If you add some new functions then feel free to add your
 * name here
 *
 * Author(s): Pablo Yanez Trujillo <yanezp@imtek.de>
 *
 */

#include <gutils.h>
#include <string.h>

/*
 * GString* g_string_ltrim(GString* string)
 *
 * Searches for the first non-empty (using isspace) character from the left side.
 * When it is found the whole string after that character is moved to the first
 * place. The str field of GString won't change the pointer.
 *
 * Returns: the GString
 */
GString* g_string_ltrim(GString* string)
{
	gchar* str;
	int i = 0, len = 0;
	
	if(!string)
		return NULL;

	str = string->str;

	while(isspace(str[i])) ++i;

	if(!i) return string;

	len = string->len - i + 1;

	memmove(str, str + i, len);

	string->len = strlen(str);

	return string;
}

/*
 * GString* g_string_rtrim(GString* string)
 *
 * Searches for the first non-empty (using isspace) character from the right side.
 * When it is found then the next character is set to '\0'.
 * The str field of GString won't change the pointer.
 *
 * Returns: the GString
 */
GString* g_string_rtrim(GString* string)
{
	gchar* str;
	int i = 0;

	if(!string) return NULL;

	str = string->str;

	i = string->len - 1;

	while(isspace(str[i])) --i;

	str[i + 1] = 0;

	string->len = strlen(str);

	return string;
}

/*
 * GString* g_string_trim(GString* string)
 *
 * Executes g_string_ltrim and then g_string_rtrim on "string"
 *
 * Returns: the GString
 */
GString* g_string_trim(GString* string)
{
	if(!string) return NULL;

	string = g_string_ltrim(string);
	string = g_string_rtrim(string);

	return string;
}


/*
 * GString* g_string_iso_to_utf8(const gchar* string)
 *
 * Converts a string in ISO-8859-1 into a one in UTF-8. It takes
 * only '\0' terminated string.
 *
 * Returns: a GString in UTF-8 format
 */
GString* g_string_iso_to_utf8(const gchar* string)
{
	GString *str;
	GError *error = NULL;
	gchar* str_utf8 = NULL;
	if(!string) return NULL;

	str_utf8 = g_convert(string, -1, "UTF-8", "ISO-8859-1", NULL, NULL, &error);

	if (error != NULL)
	{
		g_error_free (error);
		g_free(str_utf8);
		return NULL;
	}

	str = g_string_new(str_utf8);

	g_free(str_utf8);

	return str;
}

/*
 * GString* g_string_utf8_to_iso(const gchar* string)
 *
 * Converts a string in UTF-( into one in ISO-8859-1. It takes
 * only '\0' terminated string.
 *
 * Returns: a GString in ISO-8859-1 format
 */
GString* g_string_utf8_to_iso(const gchar* string)
{
	GString *str;
        GError *error = NULL;
        gchar* str_iso = NULL;
        if(!string) return NULL;

        str_iso = g_convert(string, -1, "ISO-8859-1", "UTF-8", NULL, NULL, &error);

        if (error != NULL)
        {
                g_error_free (error);
                g_free(str_iso);
                return NULL;
        }

        str = g_string_new(str_iso);

        g_free(str_iso);

        return str;
}

/*
 * void g_list_string_free(GList* list);
 *
 * Equivalent to g_list_free. 
 * g_string_fee(...,TRUE) is executed on each
 * element of the list
 */
static void __list_string_free(gpointer data, gpointer user_data)
{
        GString* str = data;

        g_string_free(str, TRUE);
}

void g_list_string_free(GList* list)
{
	if(!list) return;

	g_list_foreach(list, __list_string_free, NULL);

	g_list_free(list);
}
